@@ -21,9 +21,11 @@ import (
 	"text/template"
 	"time"
 
+	"ariga.io/atlas/cmd/atlas/internal/cmdext"
 	"ariga.io/atlas/cmd/atlas/internal/cmdlog"
 	"ariga.io/atlas/cmd/atlas/internal/lint"
 	cmdmigrate "ariga.io/atlas/cmd/atlas/internal/migrate"
+	"ariga.io/atlas/cmd/atlas/internal/migrate/ent"
 	"ariga.io/atlas/cmd/atlas/internal/migrate/ent/revision"
 	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/schema"
@@ -220,7 +222,7 @@ func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags)
 		drv migrate.Driver
 	)
 	for _, f := range pending {
-		drv, rrw, err = mux.driver(cmd.Context())
+		drv, rrw, err = mux.driverFor(cmd.Context(), f)
 		if err != nil {
 			return err
 		}
@@ -329,15 +331,15 @@ directory state to the desired schema. The desired state can be another connecte
 }
 
 func migrateDiffRun(cmd *cobra.Command, args []string, flags migrateDiffFlags) error {
-	// Open a dev driver.
-	dev, err := sqlclient.Open(cmd.Context(), flags.devURL)
+	ctx := cmd.Context()
+	dev, err := sqlclient.Open(ctx, flags.devURL)
 	if err != nil {
 		return err
 	}
 	defer dev.Close()
 	// Acquire a lock.
 	if l, ok := dev.Driver.(schema.Locker); ok {
-		unlock, err := l.Lock(cmd.Context(), "atlas_migrate_diff", flags.lockTimeout)
+		unlock, err := l.Lock(ctx, "atlas_migrate_diff", flags.lockTimeout)
 		if err != nil {
 			return fmt.Errorf("acquiring database lock: %w", err)
 		}
@@ -360,8 +362,23 @@ func migrateDiffRun(cmd *cobra.Command, args []string, flags migrateDiffFlags) e
 	if err != nil {
 		return err
 	}
+	var name string
+	if len(args) > 0 {
+		name = args[0]
+	}
+	// If there is a state-loader that requires a custom
+	// 'migrate diff' handling, offload it the work.
+	if d, ok := cmdext.States.Differ(flags.desiredURLs); ok {
+		err := d.MigrateDiff(ctx, &cmdext.MigrateDiffOptions{
+			To:   flags.desiredURLs,
+			Name: name,
+			Dir:  dir,
+			Dev:  dev,
+		})
+		return maskNoPlan(cmd, err)
+	}
 	// Get a state reader for the desired state.
-	desired, err := stateReader(cmd.Context(), &stateReaderConfig{
+	desired, err := stateReader(ctx, &stateReaderConfig{
 		urls:    flags.desiredURLs,
 		dev:     dev,
 		client:  dev,
@@ -379,31 +396,33 @@ func migrateDiffRun(cmd *cobra.Command, args []string, flags migrateDiffFlags) e
 	}
 	// Plan the changes and create a new migration file.
 	pl := migrate.NewPlanner(dev.Driver, dir, opts...)
-	var name string
-	if len(args) > 0 {
-		name = args[0]
-	}
 	plan, err := func() (*migrate.Plan, error) {
 		if dev.URL.Schema != "" {
-			return pl.PlanSchema(cmd.Context(), name, desired.StateReader)
+			return pl.PlanSchema(ctx, name, desired.StateReader)
 		}
-		return pl.Plan(cmd.Context(), name, desired.StateReader)
+		return pl.Plan(ctx, name, desired.StateReader)
 	}()
-	var cerr migrate.NotCleanError
+	var cerr *migrate.NotCleanError
 	switch {
-	case errors.Is(err, migrate.ErrNoPlan):
-		cmd.Println("The migration directory is synced with the desired state, no changes to be made")
-		return nil
 	case errors.As(err, &cerr) && dev.URL.Schema == "" && desired.schema != "":
 		return fmt.Errorf("dev database is not clean (%s). Add a schema to the URL to limit the scope of the connection", cerr.Reason)
 	case err != nil:
-		return err
+		return maskNoPlan(cmd, err)
 	default:
 		// Write the plan to a new file.
 		return pl.WritePlan(plan)
 	}
 }
 
+// maskNoPlan masks ErrNoPlan errors.
+func maskNoPlan(cmd *cobra.Command, err error) error {
+	if errors.Is(err, migrate.ErrNoPlan) {
+		cmd.Println("The migration directory is synced with the desired state, no changes to be made")
+		return nil
+	}
+	return err
+}
+
 type migrateHashFlags struct{ dirURL, dirFormat string }
 
 // migrateHashCmd represents the 'atlas migrate hash' subcommand.
@@ -687,7 +706,7 @@ func migrateNewCmd() *cobra.Command {
 				if err := dirFormatBC(flags.dirFormat, &flags.dirURL); err != nil {
 					return err
 				}
-				return checkDir(cmd, flags.dirURL, false)
+				return checkDir(cmd, flags.dirURL, true)
 			},
 			RunE: func(cmd *cobra.Command, args []string) error {
 				return migrateNewRun(cmd, args, flags)
@@ -735,14 +754,13 @@ func migrateSetCmd() *cobra.Command {
 	var (
 		flags migrateSetFlags
 		cmd   = &cobra.Command{
-			Use:   "set [flags] <version>",
+			Use:   "set [flags] [version]",
 			Short: "Set the current version of the migration history table.",
 			Long: `'atlas migrate set' edits the revision table to consider all migrations up to and including the given version
 to be applied. This command is usually used after manually making changes to the managed database.`,
 			Example: `  atlas migrate set 3 --url mysql://user:pass@localhost:3306/
-  atlas migrate set 4 --env local
+  atlas migrate set --env local
   atlas migrate set 1.2.4 --url mysql://user:pass@localhost:3306/my_db --revision-schema my_revisions`,
-			Args: cobra.ExactArgs(1),
 			PreRunE: func(cmd *cobra.Command, _ []string) error {
 				if err := migrateFlagsFromEnv(cmd); err != nil {
 					return err
@@ -765,29 +783,24 @@ to be applied. This command is usually used after manually making changes to the
 	return cmd
 }
 
-func migrateSetRun(cmd *cobra.Command, args []string, flags migrateSetFlags) error {
+func migrateSetRun(cmd *cobra.Command, args []string, flags migrateSetFlags) (rerr error) {
+	ctx := cmd.Context()
 	dir, err := dir(flags.dirURL, false)
 	if err != nil {
 		return err
 	}
-	avail, err := dir.Files()
+	files, err := dir.Files()
 	if err != nil {
 		return err
 	}
-	// Check if the target version does exist in the migration directory.
-	if idx := migrate.FilesLastIndex(avail, func(f migrate.File) bool {
-		return f.Version() == args[0]
-	}); idx == -1 {
-		return fmt.Errorf("migration with version %q not found", args[0])
-	}
-	client, err := sqlclient.Open(cmd.Context(), flags.url)
+	client, err := sqlclient.Open(ctx, flags.url)
 	if err != nil {
 		return err
 	}
 	defer client.Close()
 	// Acquire a lock.
 	if l, ok := client.Driver.(schema.Locker); ok {
-		unlock, err := l.Lock(cmd.Context(), applyLockValue, 0)
+		unlock, err := l.Lock(ctx, applyLockValue, 0)
 		if err != nil {
 			return fmt.Errorf("acquiring database lock: %w", err)
 		}
@@ -798,104 +811,132 @@ func migrateSetRun(cmd *cobra.Command, args []string, flags migrateSetFlags) err
 		return err
 	}
 	// Ensure revision table exists.
-	rrw, err := entRevisions(cmd.Context(), client, flags.revisionSchema)
+	rrw, err := entRevisions(ctx, client, flags.revisionSchema)
 	if err != nil {
 		return err
 	}
-	if err := rrw.Migrate(cmd.Context()); err != nil {
+	if err := rrw.Migrate(ctx); err != nil {
 		return err
 	}
 	// Wrap manipulation in a transaction.
-	tx, err := client.Tx(cmd.Context(), nil)
+	tx, err := client.Tx(ctx, nil)
 	if err != nil {
 		return err
 	}
-	rrw, err = entRevisions(cmd.Context(), tx.Client, flags.revisionSchema)
+	defer func() {
+		if rerr == nil {
+			rerr = tx.Commit()
+		} else if err2 := tx.Rollback(); err2 != nil {
+			rerr = fmt.Errorf("%v: %w", err2, err)
+		}
+	}()
+	rrw, err = entRevisions(ctx, tx.Client, flags.revisionSchema)
 	if err != nil {
 		return err
 	}
-	revs, err := rrw.ReadRevisions(cmd.Context())
+	revs, err := rrw.ReadRevisions(ctx)
 	if err != nil {
 		return err
 	}
-	if err := func() error {
-		for _, r := range revs {
-			// Check all existing revisions and ensure they precede the given version. If we encounter a partially
-			// applied revision, or one with errors, mark them "fixed".
-			switch {
-			// remove revision to keep linear history
-			case r.Version > args[0]:
-				if err := rrw.DeleteRevision(cmd.Context(), r.Version); err != nil {
-					return err
-				}
-			// keep, but if with error mark "fixed"
-			case r.Version == args[0] && (r.Error != "" || r.Total != r.Applied):
-				r.Type = migrate.RevisionTypeExecute | migrate.RevisionTypeResolved
-				if err := rrw.WriteRevision(cmd.Context(), r); err != nil {
-					return err
-				}
+	var version string
+	switch n := len(args); {
+	// Prevent the case where 'migrate set' is called without a version on
+	// a clean database. i.e., we allow only removing or syncing revisions.
+	case n == 0 && len(revs) > 0:
+		// Calling set without a version and an empty
+		// migration directory purges the revision table.
+		if len(files) > 0 {
+			version = files[len(files)-1].Version()
+		}
+	case n == 1:
+		// Check if the target version does exist in the migration directory.
+		if idx := migrate.FilesLastIndex(files, func(f migrate.File) bool {
+			return f.Version() == args[0]
+		}); idx == -1 {
+			return fmt.Errorf("migration with version %q not found", args[0])
+		}
+		version = args[0]
+	default:
+		return fmt.Errorf("accepts 1 arg(s), received %d", n)
+	}
+	log := &cmdlog.MigrateSet{}
+	for _, r := range revs {
+		// Check all existing revisions and ensure they precede the given version. If we encounter a partially
+		// applied revision, or one with errors, mark them "fixed".
+		switch {
+		// remove revision to keep linear history
+		case r.Version > version:
+			log.Removed(r)
+			if err := rrw.DeleteRevision(ctx, r.Version); err != nil {
+				return err
+			}
+		// keep, but if with error mark "fixed"
+		case r.Version == version && (r.Error != "" || r.Total != r.Applied):
+			log.Set(r)
+			r.Type = migrate.RevisionTypeExecute | migrate.RevisionTypeResolved
+			if err := rrw.WriteRevision(ctx, r); err != nil {
+				return err
 			}
 		}
-		revs, err = rrw.ReadRevisions(cmd.Context())
-		if err != nil {
-			return err
+	}
+	revs, err = rrw.ReadRevisions(ctx)
+	if err != nil {
+		return err
+	}
+	// If the target version succeeds the last revision, mark
+	// migrations applied, until we reach the target version.
+	var pending []migrate.File
+	switch {
+	case len(revs) == 0:
+		// Take every file until we reach target version.
+		for _, f := range files {
+			if f.Version() > version {
+				break
+			}
+			pending = append(pending, f)
 		}
-		// If the target version succeeds the last revision, mark
-		// migrations applied, until we reach the target version.
-		var pending []migrate.File
-		switch {
-		case len(revs) == 0:
-			// Take every file until we reach target version.
-			for _, f := range avail {
-				if f.Version() > args[0] {
-					break
-				}
+	case version > revs[len(revs)-1].Version:
+	loop:
+		// Take every file succeeding the last revision until we reach target version.
+		for _, f := range files {
+			switch {
+			case f.Version() <= revs[len(revs)-1].Version:
+				// Migration precedes last revision.
+			case f.Version() > version:
+				// Migration succeeds target revision.
+				break loop
+			default: // between last revision and target
 				pending = append(pending, f)
 			}
-		case args[0] > revs[len(revs)-1].Version:
-		loop:
-			// Take every file succeeding the last revision until we reach target version.
-			for _, f := range avail {
-				switch {
-				case f.Version() <= revs[len(revs)-1].Version:
-					// Migration precedes last revision.
-				case f.Version() > args[0]:
-					// Migration succeeds target revision.
-					break loop
-				default: // between last revision and target
-					pending = append(pending, f)
-				}
-			}
 		}
-		// Mark every pending file as applied.
-		sum, err := dir.Checksum()
+	}
+	// Mark every pending file as applied.
+	sum, err := dir.Checksum()
+	if err != nil {
+		return err
+	}
+	for _, f := range pending {
+		h, err := sum.SumByName(f.Name())
 		if err != nil {
 			return err
 		}
-		for _, f := range pending {
-			h, err := sum.SumByName(f.Name())
-			if err != nil {
-				return err
-			}
-			if err := rrw.WriteRevision(cmd.Context(), &migrate.Revision{
-				Version:         f.Version(),
-				Description:     f.Desc(),
-				Type:            migrate.RevisionTypeResolved,
-				ExecutedAt:      time.Now(),
-				Hash:            h,
-				OperatorVersion: operatorVersion(),
-			}); err != nil {
-				return err
-			}
+		rev := &migrate.Revision{
+			Version:         f.Version(),
+			Description:     f.Desc(),
+			Type:            migrate.RevisionTypeResolved,
+			ExecutedAt:      time.Now(),
+			Hash:            h,
+			OperatorVersion: operatorVersion(),
 		}
-		return nil
-	}(); err != nil {
-		if err2 := tx.Rollback(); err2 != nil {
-			err = fmt.Errorf("%v: %w", err2, err)
+		log.Set(rev)
+		if err := rrw.WriteRevision(ctx, rev); err != nil {
+			return err
 		}
+	}
+	if log.Current, err = rrw.CurrentRevision(ctx); err != nil && !ent.IsNotFound(err) {
 		return err
 	}
-	return tx.Commit()
+	return cmdlog.MigrateSetTemplate.Execute(cmd.OutOrStdout(), log)
 }
 
 type migrateStatusFlags struct {
@@ -1121,17 +1162,23 @@ type tx struct {
 	dryRun       bool
 	mode, schema string
 	c            *sqlclient.Client
-	tx           *sqlclient.TxClient
 	rrw          migrate.RevisionReadWriter
+	// current transaction context.
+	tx    *sqlclient.TxClient
+	txrrw migrate.RevisionReadWriter
 }
 
-// driver returns the migrate.Driver to use to execute migration statements.
-func (tx *tx) driver(ctx context.Context) (migrate.Driver, migrate.RevisionReadWriter, error) {
+// driverFor returns the migrate.Driver to use to execute migration statements.
+func (tx *tx) driverFor(ctx context.Context, f migrate.File) (migrate.Driver, migrate.RevisionReadWriter, error) {
 	if tx.dryRun {
 		// If the --dry-run flag is given we don't want to execute any statements on the database.
 		return &dryRunDriver{tx.c.Driver}, &dryRunRevisions{tx.rrw}, nil
 	}
-	switch tx.mode {
+	mode, err := tx.modeFor(f)
+	if err != nil {
+		return nil, nil, err
+	}
+	switch mode {
 	case txModeNone:
 		return tx.c.Driver, tx.rrw, nil
 	case txModeFile:
@@ -1144,11 +1191,10 @@ func (tx *tx) driver(ctx context.Context) (migrate.Driver, migrate.RevisionReadW
 		if err != nil {
 			return nil, nil, err
 		}
-		tx.rrw, err = entRevisions(ctx, tx.tx.Client, tx.schema)
-		if err != nil {
+		if tx.txrrw, err = entRevisions(ctx, tx.tx.Client, tx.schema); err != nil {
 			return nil, nil, err
 		}
-		return tx.tx.Driver, tx.rrw, nil
+		return tx.tx.Driver, tx.txrrw, nil
 	case txModeAll:
 		// In file-mode, this function is called each time a new file is executed. Since we wrap all files into one
 		// huge transaction, if there already is an opened one, use that.
@@ -1158,14 +1204,13 @@ func (tx *tx) driver(ctx context.Context) (migrate.Driver, migrate.RevisionReadW
 			if err != nil {
 				return nil, nil, err
 			}
-			tx.rrw, err = entRevisions(ctx, tx.tx.Client, tx.schema)
-			if err != nil {
+			if tx.txrrw, err = entRevisions(ctx, tx.tx.Client, tx.schema); err != nil {
 				return nil, nil, err
 			}
 		}
-		return tx.tx.Driver, tx.rrw, nil
+		return tx.tx.Driver, tx.txrrw, nil
 	default:
-		return nil, nil, fmt.Errorf("unknown tx-mode %q", tx.mode)
+		return nil, nil, fmt.Errorf("unknown tx-mode %q", mode)
 	}
 }
 
@@ -1182,7 +1227,7 @@ func (tx *tx) mayRollback(err error) error {
 // mayCommit may commit a transaction depending on the given transaction mode.
 func (tx *tx) mayCommit() error {
 	// Only commit if each file is wrapped in a transaction.
-	if !tx.dryRun && tx.mode == txModeFile {
+	if tx.tx != nil && !tx.dryRun && tx.mode == txModeFile {
 		return tx.commit()
 	}
 	return nil
@@ -1193,10 +1238,32 @@ func (tx *tx) commit() error {
 	if tx.tx == nil {
 		return nil
 	}
-	defer func() { tx.tx = nil }()
+	defer func() { tx.tx, tx.txrrw = nil, nil }()
 	return tx.tx.Commit()
 }
 
+func (tx *tx) modeFor(f migrate.File) (string, error) {
+	l, ok := f.(*migrate.LocalFile)
+	if !ok {
+		return tx.mode, nil
+	}
+	switch ds := l.Directive("txmode"); {
+	case len(ds) > 1:
+		return "", fmt.Errorf("multiple txmode values found in file %q: %q", f.Name(), ds)
+	case len(ds) == 0 || ds[0] == tx.mode:
+		return tx.mode, nil
+	case ds[0] == txModeAll:
+		return "", fmt.Errorf("txmode %q is not allowed in file directive %q", txModeAll, f.Name())
+	case ds[0] == txModeNone, ds[0] == txModeFile:
+		if tx.mode == txModeAll {
+			return "", fmt.Errorf("cannot set txmode directive to %q in %q when txmode %q is set globally", ds[0], f.Name(), txModeAll)
+		}
+		return ds[0], nil
+	default:
+		return "", fmt.Errorf("unknown txmode %q found in file directive %q", ds[0], f.Name())
+	}
+}
+
 func operatorVersion() string {
 	v, _ := parse(version)
 	return "Atlas CLI " + v
@@ -1213,12 +1280,16 @@ func dir(u string, create bool) (migrate.Dir, error) {
 
 // dirURL returns a migrate.Dir to use as migration directory. For now only local directories are supported.
 func dirURL(u *url.URL, create bool) (migrate.Dir, error) {
-	if u.Scheme != "file" {
-		return nil, fmt.Errorf("unsupported driver %q", u.Scheme)
-	}
 	path := filepath.Join(u.Host, u.Path)
-	if path == "" {
-		path = "migrations"
+	switch u.Scheme {
+	case "mem":
+		return migrate.OpenMemDir(path), nil
+	case "file":
+		if path == "" {
+			path = "migrations"
+		}
+	default:
+		return nil, fmt.Errorf("unsupported driver %q", u.Scheme)
 	}
 	fn := func() (migrate.Dir, error) { return migrate.NewLocalDir(path) }
 	switch f := u.Query().Get("format"); f {
