@@ -537,6 +537,56 @@ env "local" {
 		// Empty list is expanded to zero blocks.
 		require.EqualError(t, err, `env "local" not defined in project file`)
 	})
+
+	t.Run("TemplateDir", func(t *testing.T) {
+		var (
+			h = `
+variable "path" {
+  type = string
+}
+
+data "template_dir" "migrations" {
+  path = var.path
+  vars = {
+    Env = atlas.env
+  }
+}
+
+env "dev" {
+  url = "sqlite://${atlas.env}?mode=memory&_fk=1"
+  migration {
+    dir = data.template_dir.migrations.url
+  }
+}
+
+env "prod" {
+  url = "sqlite://${atlas.env}?mode=memory&_fk=1"
+  migration {
+    dir = data.template_dir.migrations.url
+  }
+}
+`
+			p    = t.TempDir()
+			path = filepath.Join(p, "atlas.hcl")
+		)
+		err := os.WriteFile(path, []byte(h), 0600)
+		require.NoError(t, err)
+		for _, e := range []string{"dev", "prod"} {
+			cmd := migrateCmd()
+			cmd.AddCommand(migrateApplyCmd())
+			s, err := runCmd(
+				cmd, "apply",
+				"-c", "file://"+path,
+				"--env", e,
+				"--var", "path=testdata/templatedir",
+			)
+			require.NoError(t, err)
+			require.Contains(t, s, "Migrating to version 2 (2 migrations in total):")
+			require.Contains(t, s, fmt.Sprintf("create table %s1 (c text);", e))
+			require.Contains(t, s, fmt.Sprintf("create table %s2 (c text);", e))
+			require.Contains(t, s, fmt.Sprintf("create table users_%s2 (c text);", e))
+		}
+	})
 }
 
 func TestMigrate_ApplyTxMode(t *testing.T) {
@@ -575,7 +625,7 @@ func TestMigrate_ApplyTxMode(t *testing.T) {
 				// Apply the first 2 migrations for the faulty one.
 				s, err = runCmd(
 					migrateApplyCmd(),
-					"--dir", "file://testdata/sqlitetx_2",
+					"--dir", "file://testdata/sqlitetx2",
 					"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test_2.db")),
 					"--tx-mode", mode,
 					"2",
@@ -598,7 +648,7 @@ func TestMigrate_ApplyTxMode(t *testing.T) {
 				// Apply the rest, expect it to fail due to constraint error, but only the new one is reported.
 				s, err = runCmd(
 					migrateApplyCmd(),
-					"--dir", "file://testdata/sqlitetx_2",
+					"--dir", "file://testdata/sqlitetx2",
 					"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test_2.db")),
 					"--tx-mode", mode,
 				)
@@ -610,6 +660,41 @@ func TestMigrate_ApplyTxMode(t *testing.T) {
 	}
 }
 
+func TestMigrate_ApplyTxModeDirective(t *testing.T) {
+	for _, mode := range []string{txModeNone, txModeFile} {
+		u := openSQLite(t, "")
+		_, err := runCmd(
+			migrateApplyCmd(),
+			"--dir", "file://testdata/sqlitetx3",
+			"--url", u,
+			"--tx-mode", mode,
+		)
+		require.EqualError(t, err, `sql/migrate: execute: executing statement "INSERT INTO t1 VALUES (1), (1);" from version "20220925094021": UNIQUE constraint failed: t1.a`)
+		db, err := sql.Open("sqlite3", strings.TrimPrefix(u, "sqlite://"))
+		require.NoError(t, err)
+		var n int
+		require.NoError(t, db.QueryRow("SELECT COUNT(*) FROM sqlite_master WHERE name IN ('atlas_schema_revisions', 'users', 't1')").Scan(&n))
+		require.Equal(t, 3, n)
+		require.NoError(t, db.Close())
+	}
+
+	_, err := runCmd(
+		migrateApplyCmd(),
+		"--dir", "file://testdata/sqlitetx3",
+		"--url", "sqlite://txmode?mode=memory&_fk=1",
+		"--tx-mode", txModeAll,
+	)
+	require.EqualError(t, err, `cannot set txmode directive to "none" in "20220925094021_second.sql" when txmode "all" is set globally`)
+
+	_, err = runCmd(
+		migrateApplyCmd(),
+		"--dir", "file://testdata/sqlitetx4",
+		"--url", "sqlite://txmode?mode=memory&_fk=1",
+		"--tx-mode", txModeAll,
+	)
+	require.EqualError(t, err, `unknown txmode "unknown" found in file directive "20220925094021_second.sql"`)
+}
+
 func TestMigrate_ApplyBaseline(t *testing.T) {
 	t.Run("FromFlags", func(t *testing.T) {
 		p := t.TempDir()
@@ -712,7 +797,7 @@ func TestMigrate_Diff(t *testing.T) {
 		"--dev-url", openSQLite(t, "create table t (c int);"),
 		"--to", to,
 	)
-	require.ErrorAs(t, err, &migrate.NotCleanError{})
+	require.ErrorAs(t, err, new(*migrate.NotCleanError))
 	require.ErrorContains(t, err, "found table \"t\"")
 
 	// Works (on empty directory).
@@ -792,6 +877,89 @@ func TestMigrate_StatusJSON(t *testing.T) {
 	require.Equal(t, `"sqlite3"`, s)
 }
 
+func TestMigrate_Set(t *testing.T) {
+	u := fmt.Sprintf("sqlite://file:%s?_fk=1", filepath.Join(t.TempDir(), "test.db"))
+	_, err := runCmd(
+		migrateApplyCmd(),
+		"--dir", "file://testdata/sqlite",
+		"--url", u,
+	)
+	require.NoError(t, err)
+
+	s, err := runCmd(
+		migrateSetCmd(),
+		"--dir", "file://testdata/sqlite",
+		"-u", u,
+		"20220318104614",
+	)
+	require.NoError(t, err)
+	require.Equal(t, `Current version is 20220318104614 (1 removed):
+
+  - 20220318104615 (second)
+
+`, s)
+	s, err = runCmd(
+		migrateSetCmd(),
+		"--dir", "file://testdata/sqlite",
+		"-u", u,
+		"20220318104615",
+	)
+	require.NoError(t, err)
+	require.Equal(t, `Current version is 20220318104615 (1 set):
+
+  + 20220318104615 (second)
+
+`, s)
+
+	s, err = runCmd(
+		migrateSetCmd(),
+		"--dir", "file://testdata/baseline1",
+		"-u", u,
+	)
+	require.NoError(t, err)
+	require.Equal(t, `Current version is 1 (1 set, 2 removed):
+
+  + 1 (baseline)
+  - 20220318104614 (initial)
+  - 20220318104615 (second)
+
+`, s)
+
+	s, err = runCmd(
+		migrateSetCmd(),
+		"--dir", filepath.Join("file://", t.TempDir()), // empty dir.
+		"-u", u,
+	)
+	require.NoError(t, err)
+	require.Equal(t, `All revisions deleted (1 in total):
+
+  - 1 (baseline)
+
+`, s)
+
+	// Empty database.
+	u = fmt.Sprintf("sqlite://file:%s?_fk=1", filepath.Join(t.TempDir(), "test.db"))
+	_, err = runCmd(
+		migrateSetCmd(),
+		"--dir", "file://testdata/sqlite",
+		"-u", u,
+	)
+	require.EqualError(t, err, "accepts 1 arg(s), received 0")
+
+	s, err = runCmd(
+		migrateSetCmd(),
+		"--dir", "file://testdata/sqlite",
+		"-u", u,
+		"20220318104614",
+	)
+	require.NoError(t, err)
+	require.Equal(t, `Current version is 20220318104614 (1 set):
+
+  + 20220318104614 (initial)
+
+`, s)
+}
+
 func TestMigrate_New(t *testing.T) {
 	var (
 		p = t.TempDir()
