@@ -101,9 +101,8 @@ migration.`,
 		}
 	)
 	cmd.Flags().SortFlags = false
-	cmd.Flags().StringSliceVarP(&flags.paths, flagFile, "f", nil, "[paths...] file or directory containing the HCL files")
 	addFlagURL(cmd.Flags(), &flags.url)
-	addFlagURLs(cmd.Flags(), &flags.toURLs, flagTo, "")
+	addFlagToURLs(cmd.Flags(), &flags.toURLs)
 	addFlagExclude(cmd.Flags(), &flags.exclude)
 	addFlagSchemas(cmd.Flags(), &flags.schemas)
 	addFlagDevURL(cmd.Flags(), &flags.devURL)
@@ -111,6 +110,9 @@ migration.`,
 	addFlagAutoApprove(cmd.Flags(), &flags.autoApprove)
 	cmd.Flags().StringVarP(&flags.logFormat, flagLog, "", "", "custom logging using a Go template")
 	addFlagDSN(cmd.Flags(), &flags.dsn)
+	// Hidden support for the deprecated -f flag.
+	cmd.Flags().StringSliceVarP(&flags.paths, flagFile, "f", nil, "[paths...] file or directory containing HCL or SQL files")
+	cobra.CheckErr(cmd.Flags().MarkHidden(flagFile))
 	cmd.MarkFlagsMutuallyExclusive(flagFile, flagTo)
 	return cmd
 }
@@ -181,10 +183,9 @@ func schemaApplyRun(cmd *cobra.Command, _ []string, flags schemaApplyFlags) erro
 	// Returning at this stage should
 	// not trigger the help message.
 	cmd.SilenceUsage = true
-	if len(changes) == 0 {
-		return format.Execute(cmd.OutOrStderr(), &cmdlog.SchemaApply{})
-	}
 	switch {
+	case len(changes) == 0:
+		return format.Execute(cmd.OutOrStderr(), &cmdlog.SchemaApply{})
 	case flags.logFormat != "" && flags.autoApprove:
 		var (
 			applied int
@@ -214,9 +215,7 @@ func schemaApplyRun(cmd *cobra.Command, _ []string, flags schemaApplyFlags) erro
 			return err
 		}
 		if !flags.dryRun && (flags.autoApprove || promptUser()) {
-			if err := client.ApplyChanges(ctx, changes); err != nil {
-				return err
-			}
+			return client.ApplyChanges(ctx, changes)
 		}
 		return nil
 	}
@@ -335,7 +334,7 @@ The database states can be read from a connected database, an HCL project or a m
 	)
 	cmd.Flags().SortFlags = false
 	addFlagURLs(cmd.Flags(), &flags.fromURL, flagFrom, flagFromShort)
-	addFlagURLs(cmd.Flags(), &flags.toURL, flagTo, "")
+	addFlagToURLs(cmd.Flags(), &flags.toURL)
 	addFlagDevURL(cmd.Flags(), &flags.devURL)
 	addFlagSchemas(cmd.Flags(), &flags.schemas)
 	addFlagExclude(cmd.Flags(), &flags.exclude)
@@ -393,10 +392,11 @@ func schemaDiffRun(cmd *cobra.Command, _ []string, flags schemaDiffFlags) error
 }
 
 type schemaInspectFlags struct {
-	url     string   // URL of database to apply the changes on.
-	schemas []string // Schemas to take into account when diffing.
-	exclude []string // List of glob patterns used to filter resources from applying (see schema.InspectOptions).
-	dsn     string   // Deprecated: DSN is an alias for URL.
+	url       string   // URL of database to apply the changes on.
+	logFormat string   // Format of the log output.
+	schemas   []string // Schemas to take into account when diffing.
+	exclude   []string // List of glob patterns used to filter resources from applying (see schema.InspectOptions).
+	dsn       string   // Deprecated: DSN is an alias for URL.
 }
 
 // schemaInspectCmd represents the 'atlas schema inspect' subcommand.
@@ -438,6 +438,7 @@ flag.
 	addFlagURL(cmd.Flags(), &flags.url)
 	addFlagSchemas(cmd.Flags(), &flags.schemas)
 	addFlagExclude(cmd.Flags(), &flags.exclude)
+	addFlagLog(cmd.Flags(), &flags.logFormat)
 	addFlagDSN(cmd.Flags(), &flags.dsn)
 	cobra.CheckErr(cmd.MarkFlagRequired(flagURL))
 	return cmd
@@ -457,15 +458,17 @@ func schemaInspectRun(cmd *cobra.Command, _ []string, flags schemaInspectFlags)
 		Schemas: schemas,
 		Exclude: flags.exclude,
 	})
-	if err != nil {
-		return err
+	format := cmdlog.SchemaInspectTemplate
+	if v := flags.logFormat; v != "" {
+		if format, err = template.New("format").Funcs(cmdlog.InspectTemplateFuncs).Parse(v); err != nil {
+			return fmt.Errorf("parse log format: %w", err)
+		}
 	}
-	ddl, err := client.MarshalSpec(s)
-	if err != nil {
-		return err
-	}
-	cmd.Print(string(ddl))
-	return nil
+	return format.Execute(cmd.OutOrStdout(), &cmdlog.SchemaInspect{
+		Client: client,
+		Realm:  s,
+		Error:  err,
+	})
 }
 
 // schemaFmtCmd represents the 'atlas schema fmt' subcommand.
@@ -555,15 +558,11 @@ func setSchemaEnvFlags(cmd *cobra.Command, env *Env) error {
 	if err := maySetFlag(cmd, flagFile, strings.Join(srcs, ",")); err != nil {
 		return err
 	}
-	if s := strings.Join(env.Schemas, ","); s != "" {
-		if err := maySetFlag(cmd, flagSchema, s); err != nil {
-			return err
-		}
+	if err := maySetFlag(cmd, flagSchema, strings.Join(env.Schemas, ",")); err != nil {
+		return err
 	}
-	if s := strings.Join(env.Exclude, ","); s != "" {
-		if err := maySetFlag(cmd, flagExclude, s); err != nil {
-			return err
-		}
+	if err := maySetFlag(cmd, flagExclude, strings.Join(env.Exclude, ",")); err != nil {
+		return err
 	}
 	switch cmd.Name() {
 	case "apply":
@@ -586,15 +585,13 @@ func computeDiff(ctx context.Context, differ *sqlclient.Client, from, to *stateR
 	var diff []schema.Change
 	switch {
 	// Compare realm if the desired state is an HCL file or both connections are not bound to a schema.
-	case to.hcl, from.schema == "" && to.schema == "":
+	case from.hcl, to.hcl, from.schema == "" && to.schema == "":
 		diff, err = differ.RealmDiff(current, desired)
 		if err != nil {
 			return nil, err
 		}
-	case from.schema == "":
-		return nil, fmt.Errorf("cannot diff schema %q with a database connection", from.schema)
-	case to.schema == "":
-		return nil, fmt.Errorf("cannot diff database connection with a schema %q", to.schema)
+	case from.schema == "", to.schema == "":
+		return nil, fmt.Errorf("cannot diff a schema with a database connection: %q <> %q", from.schema, to.schema)
 	default:
 		// SchemaDiff checks for name equality which is irrelevant in the case
 		// the user wants to compare their contents, reset them to allow the comparison.
@@ -629,7 +626,10 @@ func promptUser() bool {
 		Items: []string{answerApply, answerAbort},
 	}
 	_, result, err := prompt.Run()
-	cobra.CheckErr(err)
+	if err != nil && err != promptui.ErrInterrupt {
+		// Fail in case of unexpected errors.
+		cobra.CheckErr(err)
+	}
 	return result == answerApply
 }
 
