@@ -345,6 +345,7 @@ func TestSchema_ApplyLog(t *testing.T) {
 		require.NoError(t, err)
 		require.Equal(t, "{\"Pending\":[\"CREATE TABLE `t1` (`id` int NULL)\"]}", s)
 	})
+
 	t.Run("AutoApprove", func(t *testing.T) {
 		db := openSQLite(t, "")
 		cmd := schemaCmd()
@@ -412,7 +413,7 @@ func TestSchema_ApplyLog(t *testing.T) {
 	})
 }
 
-func TesSchema_ApplySources(t *testing.T) {
+func TestSchema_ApplySources(t *testing.T) {
 	var (
 		p   = t.TempDir()
 		cfg = filepath.Join(p, "atlas.hcl")
@@ -458,9 +459,9 @@ env "local" {
 	require.Equal(t, []string{
 		"-- Planned Changes:",
 		`-- Create "one" table`,
-		"CREATE TABLE `one` (`id` int NOT NULL)",
+		"CREATE TABLE `one` (`id` int NOT NULL);",
 		`-- Create "two" table`,
-		"CREATE TABLE `two` (`id` int NOT NULL)",
+		"CREATE TABLE `two` (`id` int NOT NULL);",
 	}, lines)
 }
 
