@@ -14,6 +14,7 @@ import (
 	"text/template"
 
 	"ariga.io/atlas/cmd/atlas/internal/cmdlog"
+	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/schema"
 	"ariga.io/atlas/sql/sqlclient"
 	_ "ariga.io/atlas/sql/sqlite"
@@ -130,3 +131,50 @@ func TestSchemaInspect_EncodeSQL(t *testing.T) {
 	require.NoError(t, tmpl.Execute(&b, &cmdlog.SchemaInspect{Error: errors.New("failure")}))
 	require.Equal(t, "failure", b.String())
 }
+
+func TestMigrateSet(t *testing.T) {
+	var (
+		b   bytes.Buffer
+		log = &cmdlog.MigrateSet{}
+	)
+	require.NoError(t, cmdlog.MigrateSetTemplate.Execute(&b, log))
+	require.Empty(t, b.String())
+
+	log.Current = &migrate.Revision{Version: "1"}
+	require.NoError(t, cmdlog.MigrateSetTemplate.Execute(&b, log))
+	require.Empty(t, b.String())
+
+	log.Current = &migrate.Revision{Version: "1"}
+	log.Removed(&migrate.Revision{Version: "2"})
+	log.Removed(&migrate.Revision{Version: "3", Description: "desc"})
+	require.NoError(t, cmdlog.MigrateSetTemplate.Execute(&b, log))
+	require.Equal(t, `Current version is 1 (2 removed):
+
+  - 2
+  - 3 (desc)
+
+`, b.String())
+
+	b.Reset()
+	log.Set(&migrate.Revision{Version: "1.1", Description: "desc"})
+	require.NoError(t, cmdlog.MigrateSetTemplate.Execute(&b, log))
+	require.Equal(t, `Current version is 1 (1 set, 2 removed):
+
+  + 1.1 (desc)
+  - 2
+  - 3 (desc)
+
+`, b.String())
+
+	b.Reset()
+	log.Current, log.Revisions = nil, nil
+	log.Removed(&migrate.Revision{Version: "2"})
+	log.Removed(&migrate.Revision{Version: "3", Description: "desc"})
+	require.NoError(t, cmdlog.MigrateSetTemplate.Execute(&b, log))
+	require.Equal(t, `All revisions deleted (2 in total):
+
+  - 2
+  - 3 (desc)
+
+`, b.String())
+}
