@@ -44,6 +44,11 @@ func (s *State) setInputVals(ctx *hcl.EvalContext, body hcl.Body, input map[stri
 	for _, v := range doc.Vars {
 		var vv cty.Value
 		switch iv, ok := input[v.Name]; {
+		case !v.Type.Type().IsCapsuleType():
+			return fmt.Errorf(
+				"invalid type %q for variable %q. Valid types are: string, number, bool, list, map, or set",
+				v.Type.AsString(), v.Name,
+			)
 		case ok:
 			vv = iv
 		case v.Default != cty.NilVal:
@@ -269,15 +274,33 @@ func blockVars(blocks hclsyntax.Blocks, parentAddr string, defs *blockDef) (map[
 }
 
 func addr(parentAddr, typeName, blkName, qualifier string) string {
-	var prefixDot string
-	if len(parentAddr) > 0 {
-		prefixDot = "."
+	var b strings.Builder
+	if parentAddr != "" {
+		b.WriteString(parentAddr)
+		b.WriteString(".")
 	}
-	suffix := blkName
-	if qualifier != "" {
-		suffix = qualifier + "." + blkName
+	b.WriteByte('$')
+	b.WriteString(typeName)
+	for _, s := range []string{qualifier, blkName} {
+		switch {
+		case s == "":
+		case validIdent(s):
+			b.WriteString(".")
+			b.WriteString(s)
+		default:
+			b.WriteString("[")
+			b.WriteString(strconv.Quote(s))
+			b.WriteString("]")
+		}
 	}
-	return fmt.Sprintf("%s%s$%s.%s", parentAddr, prefixDot, typeName, suffix)
+	return b.String()
+}
+
+// validIdent reports if the given string can
+// be used as an identifier in a reference.
+func validIdent(s string) bool {
+	_, err := cty.ParseNumberVal(s)
+	return err == nil || hclsyntax.ValidIdentifier(s)
 }
 
 func blockName(blk *hclsyntax.Block) (qualifier string, name string) {
