@@ -361,83 +361,6 @@ func TestMultiFile(t *testing.T) {
 	}, test.People[1])
 }
 
-func TestDynamicBlocks(t *testing.T) {
-	type (
-		URL struct {
-			Value string `spec:"value"`
-		}
-		Env struct {
-			Name string `spec:",name"`
-			URLs []*URL `spec:"url"`
-		}
-	)
-	var (
-		doc struct {
-			Envs []*Env `spec:"env"`
-		}
-		b = []byte(`
-variable "tenants" {
-  type    = list(string)
-  default = ["atlas", "ent"]
-}
-
-variable "domains" {
-  type = list(string)
-}
-
-env "prod" {
-  dynamic "url" {
-    for_each = var.tenants
-    content {
-      value = "mysql://root:pass@:3306/${url.value}"
-    }
-  }
-  migration {
-    dir = "file://migrations"
-  }
-}
-
-env "staging" {
-  dynamic "url" {
-    for_each = [for t in var.tenants: "mysql://root:pass@:3306/${t}"]
-    content {
-      value = "${url.value}"
-    }
-  }
-  migration {
-    dir = "file://migrations"
-  }
-}
-`)
-	)
-	require.NoError(t, New().EvalBytes(b, &doc, map[string]cty.Value{
-		"domains": cty.ListVal([]cty.Value{
-			cty.StringVal("a"),
-			cty.StringVal("b"),
-		}),
-	}))
-	require.Len(t, doc.Envs, 2)
-	require.Equal(t, "prod", doc.Envs[0].Name)
-	require.Equal(t, "staging", doc.Envs[1].Name)
-	require.Len(t, doc.Envs[0].URLs, 2)
-	require.Len(t, doc.Envs[1].URLs, 2)
-	require.Equal(t, "mysql://root:pass@:3306/atlas", doc.Envs[0].URLs[0].Value)
-	require.Equal(t, "mysql://root:pass@:3306/atlas", doc.Envs[1].URLs[0].Value)
-	require.Equal(t, "mysql://root:pass@:3306/ent", doc.Envs[0].URLs[1].Value)
-	require.Equal(t, "mysql://root:pass@:3306/ent", doc.Envs[1].URLs[1].Value)
-
-	// A one-element is allowed for list types.
-	require.NoError(t, New().EvalBytes(b, &doc, map[string]cty.Value{
-		"domains": cty.StringVal("a"),
-	}))
-
-	// Mismatched element types.
-	err := New().EvalBytes(b, &doc, map[string]cty.Value{
-		"domains": cty.BoolVal(false),
-	})
-	require.EqualError(t, err, `variable "domains": list of string required`)
-}
-
 func TestForEachResources(t *testing.T) {
 	type (
 		Env struct {
