@@ -156,10 +156,14 @@ func stdFuncs() map[string]function.Function {
 		"trimsuffix":      stdlib.TrimSuffixFunc,
 		"try":             tryfunc.TryFunc,
 		"upper":           stdlib.UpperFunc,
+		"urlescape":       urlEscape,
 		"urlqueryset":     urlQuerySetFunc,
 		"urlsetpath":      urlSetPathFunc,
 		"values":          stdlib.ValuesFunc,
 		"zipmap":          stdlib.ZipmapFunc,
+		// A patch from the past. Should be moved
+		// to specific scopes in the future.
+		"sql": rawExprImpl(),
 	}
 }
 
@@ -288,3 +292,17 @@ var urlSetPathFunc = function.New(&function.Spec{
 		return cty.StringVal(u.String()), nil
 	},
 })
+
+var urlEscape = function.New(&function.Spec{
+	Params: []function.Parameter{
+		{
+			Name: "string",
+			Type: cty.String,
+		},
+	},
+	Type: function.StaticReturnType(cty.String),
+	Impl: func(args []cty.Value, retType cty.Type) (cty.Value, error) {
+		u := url.QueryEscape(args[0].AsString())
+		return cty.StringVal(u), nil
+	},
+})
