@@ -201,6 +201,8 @@ func TestMemDir(t *testing.T) {
 	files, err := d.Files()
 	require.NoError(t, err)
 	require.Empty(t, files)
+	require.NoError(t, migrate.Validate(&d))
+
 	require.NoError(t, d.WriteFile("1.sql", []byte("create table t(c int);")))
 	files, err = d.Files()
 	require.NoError(t, err)
@@ -214,4 +216,86 @@ func TestMemDir(t *testing.T) {
 	hs2, err := migrate.NewHashFile(files)
 	require.NoError(t, err)
 	require.Equal(t, hs1, hs2)
+
+	// Will fail without checksum file.
+	require.ErrorIs(t, migrate.Validate(&d), migrate.ErrChecksumNotFound)
+
+	// Will not return the non-sql checksum file.
+	files, err = d.Files()
+	require.NoError(t, err)
+	require.Len(t, files, 1) // 1.sql
+}
+
+func TestOpenMemDir(t *testing.T) {
+	dev1 := migrate.OpenMemDir("dev")
+	require.NoError(t, dev1.WriteFile("1.sql", []byte("create table t1(c int);")))
+	// Open the same dir.
+	dev2 := migrate.OpenMemDir("dev")
+	files2, err := dev2.Files()
+	require.NoError(t, err)
+	require.Len(t, files2, 1)
+	require.NoError(t, dev2.WriteFile("2.sql", []byte("create table t2(c int);")))
+	files1, err := dev1.Files()
+	require.NoError(t, err)
+	require.Len(t, files1, 2)
+	files2, err = dev2.Files()
+	require.NoError(t, err)
+	require.Len(t, files2, 2)
+	// Open a new dir.
+	etc := migrate.OpenMemDir("etc")
+	files, err := etc.Files()
+	require.NoError(t, err)
+	require.Empty(t, files)
+
+	// Closing dir and opening it should not
+	// clean it if there are active references.
+	require.NoError(t, dev1.Close())
+	dev1 = migrate.OpenMemDir("dev")
+	files1, err = dev1.Files()
+	require.NoError(t, err)
+	require.Len(t, files1, 2)
+
+	// Cleanup directory on close.
+	require.NoError(t, dev1.Close())
+	require.NoError(t, dev2.Close())
+	dev1 = migrate.OpenMemDir("dev")
+	files1, err = dev1.Files()
+	require.NoError(t, err)
+	require.Empty(t, files1)
+}
+
+func TestLocalFile_Directive(t *testing.T) {
+	f := migrate.NewLocalFile("1.sql", []byte(`-- atlas:lint ignore
+alter table users drop column id;
+`))
+	require.Empty(t, f.Directive("lint"), "statement directives are ignored")
+
+	f = migrate.NewLocalFile("1.sql", []byte(`-- atlas:lint ignore
+
+alter table users drop column id;
+
+-- atlas:lint DS102
+alter table pets drop column id;
+`))
+	require.Equal(t, []string{"ignore"}, f.Directive("lint"), "single directive")
+
+	f = migrate.NewLocalFile("1.sql", []byte(`-- atlas:lint ignore
+-- atlas:txmode none
+
+alter table users drop column id;
+
+-- atlas:lint DS102
+alter table pets drop column id;
+`))
+	require.Equal(t, []string{"ignore"}, f.Directive("lint"), "first directive from two")
+	require.Equal(t, []string{"none"}, f.Directive("txmode"), "second directive from two")
+
+	f = migrate.NewLocalFile("1.sql", nil)
+	require.Empty(t, f.Directive("lint"))
+	f = migrate.NewLocalFile("1.sql", []byte("-- atlas:lint ignore"))
+	require.Empty(t, f.Directive("lint"))
+	f = migrate.NewLocalFile("1.sql", []byte("-- atlas:lint ignore\n"))
+	require.Empty(t, f.Directive("lint"))
+	f = migrate.NewLocalFile("1.sql", []byte("-- atlas:lint ignore\n\n"))
+	require.Equal(t, []string{"ignore"}, f.Directive("lint"), "double newline as directive separator")
 }
