@@ -211,6 +211,41 @@ func (d *diff) Normalize(from, to *schema.Table) error {
 	return d.defaultCharset(&to.Attrs)
 }
 
+// FindTable implements the DiffDriver.TableFinder method in order to provide
+// tables lookup that respect the "lower_case_table_names" system variable.
+func (d *diff) FindTable(s *schema.Schema, name string) (*schema.Table, error) {
+	switch d.lcnames {
+	// In mode 0: tables are stored as specified, and comparisons are case-sensitive.
+	case 0:
+		t, ok := s.Table(name)
+		if !ok {
+			return nil, &schema.NotExistError{Err: fmt.Errorf("table %q was not found", name)}
+		}
+		return t, nil
+	// In mode 1: the table are stored in lowercase, but they are still
+	// returned on inspection, because comparisons are not case-sensitive.
+	// In mode 2: the tables are stored as given but compared in lowercase.
+	// This option is not supported by Linux-based systems.
+	case 1, 2:
+		var matches []*schema.Table
+		for _, t := range s.Tables {
+			if strings.ToLower(name) == strings.ToLower(t.Name) {
+				matches = append(matches, t)
+			}
+		}
+		switch n := len(matches); n {
+		case 0:
+			return nil, &schema.NotExistError{Err: fmt.Errorf("table %q was not found", name)}
+		case 1:
+			return matches[0], nil
+		default:
+			return nil, fmt.Errorf("%d matches found for table %q", n, name)
+		}
+	default:
+		return nil, fmt.Errorf("unsupported 'lower_case_table_names' mode: %d", d.lcnames)
+	}
+}
+
 // collationChange returns the schema change for migrating the collation if
 // it was changed, and it is not the default attribute inherited from its parent.
 func (*diff) collationChange(from, top, to []schema.Attr) schema.Change {
@@ -436,7 +471,7 @@ func (d *diff) defaultChanged(from, to *schema.Column) (bool, error) {
 	case *schema.TimeType:
 		x1 := strings.ToLower(strings.Trim(d1, "' ()"))
 		x2 := strings.ToLower(strings.Trim(d2, "' ()"))
-		return x1 != x2, nil
+		return !equalsStringValues(x1, x2), nil
 	default:
 		x1 := strings.Trim(d1, "'")
 		x2 := strings.Trim(d2, "'")
