@@ -37,18 +37,6 @@ func TestDiff_TableDiff(t *testing.T) {
 			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Columns: []*schema.Column{{Name: "enum", Default: &schema.RawExpr{X: "'A'"}, Type: &schema.ColumnType{Type: &schema.EnumType{Values: []string{"A"}}}}}},
 			to:   &schema.Table{Name: "users", Columns: []*schema.Column{{Name: "enum", Default: &schema.RawExpr{X: `"A"`}, Type: &schema.ColumnType{Type: &schema.EnumType{Values: []string{"A"}}}}}},
 		},
-		{
-			name: "change primary key",
-			from: func() *schema.Table {
-				t := &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Columns: []*schema.Column{{Name: "id", Type: &schema.ColumnType{Raw: "int", Type: &schema.IntegerType{T: "int"}}}}}
-				t.PrimaryKey = &schema.Index{
-					Parts: []*schema.IndexPart{{C: t.Columns[0]}},
-				}
-				return t
-			}(),
-			to:      &schema.Table{Name: "users"},
-			wantErr: true,
-		},
 		{
 			name: "modify counter",
 			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&AutoIncrement{V: 1}}},
@@ -426,6 +414,65 @@ func TestDiff_TableDiff(t *testing.T) {
 				},
 			}
 		}(),
+		func() testcase {
+			from := schema.NewTable("t1").
+				SetSchema(schema.New("test")).
+				AddColumns(schema.NewIntColumn("id", "int"))
+			to := schema.NewTable("t1").
+				SetSchema(schema.New("test")).
+				AddColumns(schema.NewIntColumn("id", "int"))
+			to.SetPrimaryKey(schema.NewPrimaryKey(to.Columns...))
+			return testcase{
+				name: "add primary-key",
+				from: from,
+				to:   to,
+				wantChanges: []schema.Change{
+					&schema.AddPrimaryKey{P: to.PrimaryKey},
+				},
+			}
+		}(),
+		func() testcase {
+			from := schema.NewTable("t1").
+				SetSchema(schema.New("test")).
+				AddColumns(schema.NewIntColumn("id", "int"))
+			from.SetPrimaryKey(schema.NewPrimaryKey(from.Columns...))
+			to := schema.NewTable("t1").
+				SetSchema(schema.New("test")).
+				AddColumns(schema.NewIntColumn("id", "int"))
+			return testcase{
+				name: "drop primary-key",
+				from: from,
+				to:   to,
+				wantChanges: []schema.Change{
+					&schema.DropPrimaryKey{P: from.PrimaryKey},
+				},
+			}
+		}(),
+		func() testcase {
+			from := schema.NewTable("t1").
+				SetSchema(schema.New("test")).
+				AddColumns(schema.NewStringColumn("id", "varchar(255)"))
+			from.SetPrimaryKey(schema.NewPrimaryKey(from.Columns...))
+			to := schema.NewTable("t1").
+				SetSchema(schema.New("test")).
+				AddColumns(schema.NewStringColumn("id", "varchar(255)"))
+			to.SetPrimaryKey(
+				schema.NewPrimaryKey(from.Columns...).
+					AddAttrs(&IndexType{T: IndexTypeHash}),
+			)
+			return testcase{
+				name: "modify primary-key",
+				from: from,
+				to:   to,
+				wantChanges: []schema.Change{
+					&schema.ModifyPrimaryKey{
+						From:   from.PrimaryKey,
+						To:     to.PrimaryKey,
+						Change: schema.ChangeAttr,
+					},
+				},
+			}
+		}(),
 		func() testcase {
 			var (
 				ref = &schema.Table{
@@ -557,6 +604,54 @@ func TestDiff_SchemaDiff(t *testing.T) {
 	}, changes)
 }
 
+func TestDiff_LowerCaseMode(t *testing.T) {
+	db, m, err := sqlmock.New()
+	require.NoError(t, err)
+	mock{m}.lcmode("8.0.19", "1")
+	drv, err := Open(db)
+	require.NoError(t, err)
+	changes, err := drv.SchemaDiff(
+		schema.New("public").AddTables(schema.NewTable("t"), schema.NewTable("S")),
+		schema.New("public").AddTables(schema.NewTable("T"), schema.NewTable("s")),
+	)
+	require.NoError(t, err)
+	require.Nil(t, changes)
+
+	changes, err = drv.SchemaDiff(
+		schema.New("public").AddTables(schema.NewTable("t")),
+		schema.New("public").AddTables(schema.NewTable("T"), schema.NewTable("t")),
+	)
+	require.EqualError(t, err, `2 matches found for table "t"`)
+	require.Nil(t, changes)
+
+	changes, err = drv.SchemaDiff(
+		schema.New("public").AddTables(schema.NewTable("t")),
+		schema.New("public").AddTables(schema.NewTable("s"), schema.NewTable("S")),
+	)
+	require.NoError(t, err)
+	require.Len(t, changes, 3)
+
+	mock{m}.lcmode("8.0.19", "2")
+	drv, err = Open(db)
+	require.NoError(t, err)
+	changes, err = drv.SchemaDiff(
+		schema.New("public").AddTables(schema.NewTable("t"), schema.NewTable("S")),
+		schema.New("public").AddTables(schema.NewTable("T"), schema.NewTable("s")),
+	)
+	require.NoError(t, err)
+	require.Nil(t, changes)
+
+	mock{m}.lcmode("8.0.19", "3")
+	drv, err = Open(db)
+	require.NoError(t, err)
+	changes, err = drv.SchemaDiff(
+		schema.New("public").AddTables(schema.NewTable("t"), schema.NewTable("S")),
+		schema.New("public").AddTables(schema.NewTable("T"), schema.NewTable("s")),
+	)
+	require.EqualError(t, err, `unsupported 'lower_case_table_names' mode: 3`)
+	require.Nil(t, changes)
+}
+
 func TestDiff_RealmDiff(t *testing.T) {
 	db, m, err := sqlmock.New()
 	require.NoError(t, err)
