@@ -7,6 +7,7 @@ package mysql
 import (
 	"context"
 	"database/sql"
+	"net/url"
 	"testing"
 	"time"
 
@@ -18,6 +19,13 @@ import (
 	"github.com/stretchr/testify/require"
 )
 
+func TestParser_ParseURLParseTime(t *testing.T) {
+	u, err := url.Parse("mysql://user:pass@localhost:3306/my_db?foo=bar")
+	require.NoError(t, err)
+	ac := parser{}.ParseURL(u)
+	require.Equal(t, "true", ac.Query().Get("parseTime"))
+}
+
 func TestDriver_LockAcquired(t *testing.T) {
 	db, m, err := sqlmock.New()
 	require.NoError(t, err)
@@ -163,6 +171,18 @@ func TestDriver_CheckClean(t *testing.T) {
 	require.EqualError(t, err, `sql/migrate: connected database is not clean: found multiple tables: 2`)
 }
 
+func TestDriver_Version(t *testing.T) {
+	db, m, err := sqlmock.New()
+	require.NoError(t, err)
+	mock{m}.version("8.0.13")
+	drv, err := Open(db)
+	require.NoError(t, err)
+
+	type vr interface{ Version() string }
+	require.Implements(t, (*vr)(nil), drv)
+	require.Equal(t, "8.0.13", drv.(vr).Version())
+}
+
 type mockInspector struct {
 	schema.Inspector
 	realm  *schema.Realm
