@@ -160,7 +160,7 @@ func (s *state) addTable(ctx context.Context, add *schema.AddTable) error {
 		})
 		if pk := add.T.PrimaryKey; pk != nil {
 			b.Comma().P("PRIMARY KEY")
-			if err := s.indexParts(b, pk); err != nil {
+			if err := s.index(b, pk); err != nil {
 				errs = append(errs, err.Error())
 			}
 		}
@@ -200,7 +200,21 @@ func (s *state) addTable(ctx context.Context, add *schema.AddTable) error {
 
 // dropTable builds and executes the query for dropping a table from a schema.
 func (s *state) dropTable(ctx context.Context, drop *schema.DropTable) error {
-	rs := &state{conn: s.conn, PlanOptions: s.PlanOptions}
+	cmd := &changeGroup{}
+	for _, e := range s.enumTypes(drop.T) {
+		if err := s.mayDropEnum(cmd, drop.T.Schema, e, drop.T); err != nil {
+			return err
+		}
+	}
+	rs := &state{
+		conn:        s.conn,
+		PlanOptions: s.PlanOptions,
+		altered:     s.altered,
+		// Enums that were dropped above, were
+		// also created in the reverse commands.
+		created: s.dropped,
+		dropped: s.created,
+	}
 	if err := rs.addTable(ctx, &schema.AddTable{T: drop.T}); err != nil {
 		return fmt.Errorf("calculate reverse for drop table %q: %w", drop.T.Name, err)
 	}
@@ -209,7 +223,7 @@ func (s *state) dropTable(ctx context.Context, drop *schema.DropTable) error {
 		b.P("IF EXISTS")
 	}
 	b.Table(drop.T)
-	s.append(&migrate.Change{
+	cmd.main = &migrate.Change{
 		Cmd:     b.String(),
 		Source:  drop,
 		Comment: fmt.Sprintf("drop %q table", drop.T.Name),
@@ -225,7 +239,8 @@ func (s *state) dropTable(ctx context.Context, drop *schema.DropTable) error {
 			}
 			return cmd
 		}(),
-	})
+	}
+	cmd.append(s)
 	return nil
 }
 
@@ -367,7 +382,7 @@ func (s *state) alterTable(t *schema.Table, changes []schema.Change) error {
 		reverse    []schema.Change
 		reversible = true
 	)
-	build := func(alter *alterChange, changes []schema.Change) (string, error) {
+	build := func(alter *changeGroup, changes []schema.Change) (string, error) {
 		b := s.Build("ALTER TABLE").Table(t)
 		err := b.MapCommaErr(changes, func(i int, b *sqlx.Builder) error {
 			switch change := changes[i].(type) {
@@ -406,15 +421,31 @@ func (s *state) alterTable(t *schema.Table, changes []schema.Change) error {
 					}
 				}
 			case *schema.AddIndex:
+				// Skip reversing this operation as it is the inverse of
+				// the operation above and should not be used besides this.
 				b.P("ADD CONSTRAINT").Ident(change.I.Name).P("UNIQUE")
 				if err := s.indexParts(b, change.I); err != nil {
 					return err
 				}
-				// Skip reversing this operation as it is the inverse of
-				// the operation below and should not be used besides this.
 			case *schema.DropIndex:
 				b.P("DROP CONSTRAINT").Ident(change.I.Name)
 				reverse = append(reverse, &schema.AddIndex{I: change.I})
+			case *schema.AddPrimaryKey:
+				b.P("ADD PRIMARY KEY")
+				if err := s.index(b, change.P); err != nil {
+					return err
+				}
+				reverse = append(reverse, &schema.DropPrimaryKey{P: change.P})
+			case *schema.DropPrimaryKey:
+				b.P("DROP CONSTRAINT").Ident(pkName(t, change.P))
+				reverse = append(reverse, &schema.AddPrimaryKey{P: change.P})
+			case *schema.ModifyPrimaryKey:
+				b.P("DROP CONSTRAINT").Ident(pkName(t, change.From))
+				b.P(", ADD PRIMARY KEY")
+				if err := s.index(b, change.To); err != nil {
+					return err
+				}
+				reverse = append(reverse, &schema.ModifyPrimaryKey{From: change.To, To: change.From, Change: change.Change})
 			case *schema.AddForeignKey:
 				b.P("ADD")
 				s.fks(b, change.F)
@@ -458,7 +489,7 @@ func (s *state) alterTable(t *schema.Table, changes []schema.Change) error {
 		}
 		return b.String(), nil
 	}
-	cmd := &alterChange{}
+	cmd := &changeGroup{}
 	stmt, err := build(cmd, changes)
 	if err != nil {
 		return fmt.Errorf("alter table %q: %v", t.Name, err)
@@ -475,7 +506,7 @@ func (s *state) alterTable(t *schema.Table, changes []schema.Change) error {
 		// Changes should be reverted in
 		// a reversed order they were created.
 		sqlx.ReverseChanges(reverse)
-		if cmd.main.Reverse, err = build(&alterChange{}, reverse); err != nil {
+		if cmd.main.Reverse, err = build(&changeGroup{}, reverse); err != nil {
 			return fmt.Errorf("reverse alter table %q: %v", t.Name, err)
 		}
 	}
@@ -483,20 +514,20 @@ func (s *state) alterTable(t *schema.Table, changes []schema.Change) error {
 	return nil
 }
 
-// alterChange describes an alter table migrate.Change where its main command
+// changeGroup describes an alter table migrate.Change where its main command
 // can be supported by additional statements before and after it is executed.
-type alterChange struct {
+type changeGroup struct {
 	main          *migrate.Change
 	before, after []*migrate.Change
 }
 
-func (a *alterChange) append(s *state) {
+func (a *changeGroup) append(s *state) {
 	s.append(a.before...)
 	s.append(a.main)
 	s.append(a.after...)
 }
 
-func (s *state) alterColumn(b *sqlx.Builder, alter *alterChange, t *schema.Table, c *schema.ModifyColumn) error {
+func (s *state) alterColumn(b *sqlx.Builder, alter *changeGroup, t *schema.Table, c *schema.ModifyColumn) error {
 	for k := c.Change; !k.Is(schema.NoChange); {
 		b.P("ALTER COLUMN").Ident(c.To.Name)
 		switch {
@@ -552,7 +583,7 @@ func (s *state) alterColumn(b *sqlx.Builder, alter *alterChange, t *schema.Table
 
 // alterType appends the clause(s) to alter the column type and assuming the
 // "ALTER COLUMN <Name>" was called before by the alterColumn function.
-func (s *state) alterType(b *sqlx.Builder, alter *alterChange, t *schema.Table, c *schema.ModifyColumn) error {
+func (s *state) alterType(b *sqlx.Builder, alter *changeGroup, t *schema.Table, c *schema.ModifyColumn) error {
 	// Commands for creating and dropping serial sequences.
 	createDropSeq := func(st *SerialType) (string, string, string) {
 		seq := fmt.Sprintf(`%s%q`, s.schemaPrefix(t.Schema), st.sequence(t, c.To))
@@ -782,8 +813,9 @@ func (s *state) enumExists(ctx context.Context, ns *schema.Schema, e *schema.Enu
 	return rows.Next(), rows.Err()
 }
 
-// mayDropEnum drops dangling enum types form the schema.
-func (s *state) mayDropEnum(alter *alterChange, ns *schema.Schema, e *schema.EnumType) error {
+// mayDropEnum drops dangling enum types from the schema. An optional
+// "dropped" list can be provided to skip while searching for usage.
+func (s *state) mayDropEnum(cmd *changeGroup, ns *schema.Schema, e *schema.EnumType, dropped ...*schema.Table) error {
 	name := s.enumIdent(ns, e)
 	if _, ok := s.dropped[name]; ok {
 		return nil
@@ -795,6 +827,10 @@ func (s *state) mayDropEnum(alter *alterChange, ns *schema.Schema, e *schema.Enu
 	}
 	for i := range schemas {
 		for _, t := range schemas[i].Tables {
+			// Skip dropped tables.
+			if containsT(dropped, t) {
+				continue
+			}
 			for _, c := range t.Columns {
 				e1, ok := hasEnumType(c)
 				// Although we search in siblings schemas, use the
@@ -807,7 +843,7 @@ func (s *state) mayDropEnum(alter *alterChange, ns *schema.Schema, e *schema.Enu
 	}
 	s.dropped[name] = e
 	create, drop := s.createDropEnum(ns, e)
-	alter.after = append(alter.after, &migrate.Change{
+	cmd.after = append(cmd.after, &migrate.Change{
 		Cmd:     drop,
 		Reverse: create,
 		Comment: fmt.Sprintf("drop enum type %q", e.T),
@@ -1205,6 +1241,21 @@ func (s *state) enumSchema(ns *schema.Schema, e *schema.EnumType) (es string) {
 	return
 }
 
+// enumType returns all the enum types used by the given table.
+func (s *state) enumTypes(t *schema.Table) []*schema.EnumType {
+	var (
+		es   []*schema.EnumType
+		seen = make(map[string]bool)
+	)
+	for _, c := range t.Columns {
+		if e, ok := hasEnumType(c); ok && !seen[s.enumIdent(t.Schema, e)] {
+			seen[s.enumIdent(t.Schema, e)] = true
+			es = append(es, e)
+		}
+	}
+	return es
+}
+
 // schemaPrefix returns the schema prefix based on the planner config.
 func (s *state) schemaPrefix(ns *schema.Schema) string {
 	switch {
@@ -1243,3 +1294,21 @@ func hasEnumType(c *schema.Column) (*schema.EnumType, bool) {
 	}
 	return nil, false
 }
+
+func containsT(ts []*schema.Table, t *schema.Table) bool {
+	for _, t1 := range ts {
+		if t1.Schema.Name == t.Schema.Name && t1.Name == t.Name {
+			return true
+		}
+	}
+	return false
+}
+
+func pkName(t *schema.Table, pk *schema.Index) string {
+	if pk.Name != "" {
+		return pk.Name
+	}
+	// The default naming for primary-key constraints is <Table>_pkey.
+	// See: the ChooseIndexName function in PostgreSQL for more reference.
+	return t.Name + "_pkey"
+}
