@@ -12,6 +12,7 @@ import (
 	"ariga.io/atlas/sql/internal/spectest"
 	"ariga.io/atlas/sql/internal/sqlx"
 	"ariga.io/atlas/sql/schema"
+	"github.com/hashicorp/hcl/v2/hclparse"
 	"github.com/stretchr/testify/require"
 )
 
@@ -886,7 +887,6 @@ func TestMarshalSpec_IndexInclude(t *testing.T) {
 		},
 	}
 	s.Tables[0].Schema = s
-	s.Tables[0].Schema = s
 	s.Tables[0].Indexes = []*schema.Index{
 		{
 			Name:  "index",
@@ -922,6 +922,78 @@ schema "test" {
 	require.EqualValues(t, expected, string(buf))
 }
 
+func TestMarshalSpec_PrimaryKey(t *testing.T) {
+	s := schema.New("test").
+		AddTables(
+			schema.NewTable("users").
+				AddColumns(
+					schema.NewIntColumn("c", "int"),
+					schema.NewIntColumn("d", "int"),
+				),
+		)
+	s.Tables[0].SetPrimaryKey(
+		schema.NewPrimaryKey(s.Tables[0].Columns[:1]...).
+			AddAttrs(&IndexInclude{Columns: s.Tables[0].Columns[1:]}),
+	)
+	buf, err := MarshalSpec(s, hclState)
+	require.NoError(t, err)
+	const expected = `table "users" {
+  schema = schema.test
+  column "c" {
+    null = false
+    type = int
+  }
+  column "d" {
+    null = false
+    type = int
+  }
+  primary_key {
+    columns = [column.c]
+    include = [column.d]
+  }
+}
+schema "test" {
+}
+`
+	require.EqualValues(t, expected, string(buf))
+}
+
+func TestUnmarshalSpec_PrimaryKey(t *testing.T) {
+	f := `
+schema "s" {}
+table "t" {
+	schema = schema.s
+	column "c" {
+		type = int
+	}
+	column "d" {
+		type = int
+	}
+	primary_key {
+		columns = [
+			column.c,
+		]
+		include = [
+			column.d,
+		]
+	}
+}
+`
+	var s schema.Schema
+	err := EvalHCLBytes([]byte(f), &s, nil)
+	require.NoError(t, err)
+	require.Len(t, s.Tables, 1)
+	require.Len(t, s.Tables[0].Columns, 2)
+	pk := s.Tables[0].PrimaryKey
+	require.NotNil(t, pk)
+	require.Len(t, pk.Parts, 1)
+	require.Len(t, pk.Attrs, 1)
+	var include IndexInclude
+	require.True(t, sqlx.Has(pk.Attrs, &include))
+	require.Len(t, include.Columns, 1)
+	require.Equal(t, "d", include.Columns[0].Name)
+}
+
 func TestMarshalSpec_GeneratedColumn(t *testing.T) {
 	s := schema.New("test").
 		AddTables(
@@ -1447,6 +1519,18 @@ func TestTypes(t *testing.T) {
 			typeExpr: `sql("varchar(2) [ 2 ] [  ]")`,
 			expected: &ArrayType{Type: &schema.StringType{T: "varchar", Size: 2}, T: "varchar(2)[]"},
 		},
+		{
+			typeExpr: "oid",
+			expected: &OIDType{T: typeOID},
+		},
+		{
+			typeExpr: "regclass",
+			expected: &OIDType{T: typeRegClass},
+		},
+		{
+			typeExpr: "name",
+			expected: &schema.StringType{T: typeName},
+		},
 	} {
 		t.Run(tt.typeExpr, func(t *testing.T) {
 			var test schema.Schema
@@ -1737,3 +1821,52 @@ schema "s2" {
 `,
 		string(got))
 }
+
+func TestEvalHCLError(t *testing.T) {
+	for _, tt := range []struct {
+		text, err string
+	}{
+		{
+			text: `
+			table "test" {
+			  column "id" {
+			    type = string
+			  }
+			}`,
+			err: `schema.hcl:4,15-21: Unknown column.type; There is no type named "string".`,
+		},
+		{
+			text: `
+			table "test" {
+			  index "i" {
+			    type = unknown
+			  }
+			}`,
+			err: `schema.hcl:4,15-22: Unknown index.type; There is no type named "unknown".`,
+		},
+		{
+			text: `
+			table "test" {
+			  foreign_key "f" {
+			    on_delete = unknown
+			  }
+			}`,
+			err: `schema.hcl:4,20-27: Unknown foreign_key.on_delete; There is no on_delete named "unknown".`,
+		},
+		{
+			text: `
+			table "test" {
+			  partition {
+			    type = UNKNOWN
+			  }
+			}`,
+			err: `schema.hcl:4,15-22: Unknown partition.type; There is no type named "UNKNOWN".`,
+		},
+	} {
+		p := hclparse.NewParser()
+		_, diags := p.ParseHCL([]byte(tt.text), "schema.hcl")
+		require.False(t, diags.HasErrors())
+		err := EvalHCL.Eval(p, &schema.Schema{}, nil)
+		require.EqualError(t, err, tt.err)
+	}
+}
