@@ -87,9 +87,9 @@ type (
 
 // Schema returns the first schema that matched the given name.
 func (r *Realm) Schema(name string) (*Schema, bool) {
-	for _, t := range r.Schemas {
-		if t.Name == name {
-			return t, true
+	for _, s := range r.Schemas {
+		if s.Name == name {
+			return s, true
 		}
 	}
 	return nil, false
@@ -189,12 +189,13 @@ type (
 	EnumType struct {
 		T      string   // Optional type.
 		Values []string // Enum values.
+		Schema *Schema  // Optional schema.
 	}
 
 	// BinaryType represents a type that stores a binary data.
 	BinaryType struct {
 		T    string
-		Size int
+		Size *int
 	}
 
 	// StringType represents a string type.
@@ -246,6 +247,11 @@ type (
 		T string
 	}
 
+	// A UUIDType defines a UUID type.
+	UUIDType struct {
+		T string
+	}
+
 	// UnsupportedType represents a type that is not supported by the drivers.
 	UnsupportedType struct {
 		T string
@@ -320,6 +326,7 @@ func (*FloatType) typ()       {}
 func (*StringType) typ()      {}
 func (*BinaryType) typ()      {}
 func (*SpatialType) typ()     {}
+func (*UUIDType) typ()        {}
 func (*IntegerType) typ()     {}
 func (*DecimalType) typ()     {}
 func (*UnsupportedType) typ() {}
