@@ -14,8 +14,13 @@ import (
 	"ariga.io/atlas/sql/schema"
 )
 
+// DefaultDiff provides basic diffing capabilities for MySQL dialects.
+// Note, it is recommended to call Open, create a new Driver and use its
+// Differ when a database connection is available.
+var DefaultDiff schema.Differ = &sqlx.Diff{DiffDriver: &diff{}}
+
 // A diff provides a SQLite implementation for sqlx.DiffDriver.
-type diff struct{ conn }
+type diff struct{}
 
 // SchemaAttrDiff returns a changeset for migrating schema attributes from one state to the other.
 func (d *diff) SchemaAttrDiff(_, _ *schema.Schema) []schema.Change {
@@ -40,7 +45,7 @@ func (d *diff) TableAttrDiff(from, to *schema.Table) ([]schema.Change, error) {
 }
 
 // ColumnChange returns the schema changes (if any) for migrating one column to the other.
-func (d *diff) ColumnChange(from, to *schema.Column) (schema.ChangeKind, error) {
+func (d *diff) ColumnChange(_ *schema.Table, from, to *schema.Column) (schema.ChangeKind, error) {
 	change := sqlx.CommentChange(from.Attrs, to.Attrs)
 	if from.Type.Null != to.Type.Null {
 		change |= schema.ChangeNull
@@ -113,7 +118,7 @@ func (*diff) IndexAttrChanged(from, to []schema.Attr) bool {
 }
 
 // IndexPartAttrChanged reports if the index-part attributes were changed.
-func (*diff) IndexPartAttrChanged(_, _ *schema.IndexPart) bool {
+func (*diff) IndexPartAttrChanged(_, _ *schema.Index, _ int) bool {
 	return false
 }
 
@@ -131,10 +136,10 @@ func (*diff) ReferenceChanged(from, to schema.ReferenceOption) bool {
 }
 
 // Normalize implements the sqlx.Normalizer interface.
-func (d *diff) Normalize(from, to *schema.Table) {
+func (d *diff) Normalize(from, to *schema.Table) error {
 	used := make([]bool, len(to.ForeignKeys))
 	// In SQLite, there is no easy way to get the foreign-key constraint
-	// name, except for parsing the CREATE statement). Therefore, we check
+	// name, except for parsing the CREATE statement. Therefore, we check
 	// if there is a foreign-key with identical properties.
 	for _, fk1 := range from.ForeignKeys {
 		for i, fk2 := range to.ForeignKeys {
@@ -147,6 +152,7 @@ func (d *diff) Normalize(from, to *schema.Table) {
 			}
 		}
 	}
+	return nil
 }
 
 func sameFK(fk1, fk2 *schema.ForeignKey) bool {
