@@ -8,7 +8,6 @@ import (
 	"context"
 	"testing"
 
-	"ariga.io/atlas/sql/internal/sqlx"
 	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/schema"
 	"ariga.io/atlas/sql/sqlcheck"
@@ -26,7 +25,7 @@ func TestDetectModifyTable(t *testing.T) {
 			Dev: &sqlclient.Client{
 				Driver: func() migrate.Driver {
 					drv := &sqlite.Driver{}
-					drv.Differ = &sqlx.Diff{DiffDriver: &sqlite.Diff{}}
+					drv.Differ = sqlite.DefaultDiff
 					return drv
 				}(),
 			},
@@ -77,7 +76,7 @@ func TestDetectModifyTable(t *testing.T) {
 								T: schema.NewTable("posts").
 									SetSchema(schema.New("main")).
 									AddColumns(
-										schema.NewStringColumn("text", "text"),
+										schema.NewNullStringColumn("text", "text"),
 										schema.NewTimeColumn("posted_at", "timestamp"),
 									),
 							},
@@ -126,7 +125,9 @@ func TestDetectModifyTable(t *testing.T) {
 	)
 	azs, err := sqlcheck.AnalyzerFor(sqlite.DriverName, nil)
 	require.NoError(t, err)
-	err = sqlcheck.Analyzers(azs).Analyze(context.Background(), pass)
+	require.Len(t, azs, 4)
+	require.NoError(t, azs[0].Analyze(context.Background(), pass))
+	err = azs[1].Analyze(context.Background(), pass)
 	require.EqualError(t, err, "destructive changes detected")
 
 	require.Equal(t, report.Text, "destructive changes detected")
@@ -134,6 +135,11 @@ func TestDetectModifyTable(t *testing.T) {
 	require.Equal(t, report.Diagnostics[0].Text, `Dropping table "users"`)
 	require.Equal(t, report.Diagnostics[1].Text, `Dropping non-virtual column "posted_at"`)
 	require.Equal(t, report.Diagnostics[2].Text, `Dropping table "pets"`)
+
+	require.NoError(t, azs[2].Analyze(context.Background(), pass))
+	require.Equal(t, report.Text, "data dependent changes detected")
+	require.Len(t, report.Diagnostics, 1)
+	require.Equal(t, report.Diagnostics[0].Text, `Modifying nullable column "text" to non-nullable without default value might fail in case it contains NULL values`)
 }
 
 type testFile struct {
